/*******************************************************************************
***  This file is part of Dux Rei Nummariae.                                 ***
***                                                                          ***
***  Copyright (C) 2017, 2018, 2019, 2020, 2021, 2022, 2023                  ***
***  John Schneiderman <Licencing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU Affero General Public License as published   ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU Affero General Public License for more details.             ***
***                                                                          ***
***  You should have received a copy of the GNU Affero General Public License***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*******************************************************************************/
#ifndef DRN_ACCOUNTING_BALANCEADJUSTMENTS_H_
#define DRN_ACCOUNTING_BALANCEADJUSTMENTS_H_

#include "accounting_export.h"


namespace pecunia
{
namespace currency
{

class Money;

}}

namespace drn
{
namespace accounting
{

class AccountCode;
class Transaction;

/**
 * @brief Performs a theoretical adjustment of a supplied balance according to accounting rules.
 *
 * @exception AccountError When neither the crediting nor debiting accounts is the requested asset
 * account.
 *
 * @param asset The code for the asset account to adjust.
 * @param balance The balance to start with in the adjustment.
 * @param txn The transaction to apply to the balance.
 *
 * @return The calculated balance after applying a transaction.
 */
DRN_ACCOUNTING_EXPORT pecunia::currency::Money adjustAssetBalance(
	const AccountCode& asset,
	const pecunia::currency::Money& balance,
	const Transaction& txn
);

}}

#endif
