/*******************************************************************************
***  This file is part of Dux Rei Nummariae.                                 ***
***                                                                          ***
***  Copyright (C) 2017, 2018, 2019, 2020, 2021, 2022, 2023                  ***
***  John Schneiderman <Licencing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU Affero General Public License as published   ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU Affero General Public License for more details.             ***
***                                                                          ***
***  You should have received a copy of the GNU Affero General Public License***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*******************************************************************************/
#ifndef DRN_NAVIGATION_USERPREFERENCES_H_
#define DRN_NAVIGATION_USERPREFERENCES_H_

#include <QSettings>

#include <chrono>
#include <set>

#include <foundation/CalendarTime.hpp>


namespace pecunia
{
namespace currency
{

enum class Iso4217Codes : quint16;

}}

namespace drn
{
namespace navigation
{

/**
 * @brief The representation for the setting values the users prefers.
 */
class UserPreferences
{
	QSettings settings_;

public:
	UserPreferences();

	//{ budgeting Preferences

	std::chrono::hours hoursInWeek() const;
	void setHoursInWeek(const std::chrono::hours& value);
	foundation::Days daysInWeek() const;
	void setDaysInWeek(const foundation::Days& value);
	foundation::Days notifyDueWithin() const;
	void setNotifyDueWithin(const foundation::Days& value);

	//}

	//{ currency Preferences

	pecunia::currency::Iso4217Codes preferredCurrency() const;
	void setPreferredCurrency(const pecunia::currency::Iso4217Codes& value);
	std::set<pecunia::currency::Iso4217Codes> usableCurrencies() const;
	void setUsableCurrencies(const std::set<pecunia::currency::Iso4217Codes>& values);

	//}
};

}}

#endif
