/*******************************************************************************
***  This file is part of Dux Rei Nummariae.                                 ***
***                                                                          ***
***  Copyright (C) 2017, 2018, 2019, 2020, 2021, 2022, 2023                  ***
***  John Schneiderman <Licencing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU Affero General Public License as published   ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU Affero General Public License for more details.             ***
***                                                                          ***
***  You should have received a copy of the GNU Affero General Public License***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*******************************************************************************/
#ifndef DRN_NETWORK_NETWORKERRORS_H_
#define DRN_NETWORK_NETWORKERRORS_H_

#include <foundation/Error.h>
#include "network_export.h"


class QNetworkReply;
class QString;

namespace drn
{
namespace network
{

/**
 * @brief Represents an error with a network reply.
 */
struct DRN_NETWORK_EXPORT ReplyError : foundation::Error
{
	/**
	 * @brief Full initialisation constructor.
	 *
	 * @param reply The reply that has the error.
	 * @param details Any additional details for the error message.
	 * @param root Any root exception triggering the error.
	 */
	ReplyError(
		const QNetworkReply& reply,
		const QString& details,
		const std::exception& root = {}
	);
	ReplyError(
		const QNetworkReply&,
		const char* details,
		const std::exception& = {}
	) = delete;
};

}}

#endif
