/*******************************************************************************
***  This file is part of Dux Rei Nummariae.                                 ***
***                                                                          ***
***  Copyright (C) 2017, 2018, 2019, 2020, 2021, 2022, 2023                  ***
***  John Schneiderman <Licencing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU Affero General Public License as published   ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU Affero General Public License for more details.             ***
***                                                                          ***
***  You should have received a copy of the GNU Affero General Public License***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*******************************************************************************/
#ifndef DRN_SURVEYING_BANKACCOUNTSURVEY_H_
#define DRN_SURVEYING_BANKACCOUNTSURVEY_H_

#include <pecunia/Money.h>

#include <map>
#include <set>

#include <accounting/Transaction.h>
#include <banking/Reconciliation.h>
#include <budgeting/BudgetItemIdentifier.h>
#include "surveying_export.h"


namespace drn
{
namespace surveying
{

class DRN_SURVEYING_EXPORT BankAccountSurvey
{
	banking::ReconciledBankAccount reconciled_;
	pecunia::currency::Money projectedBalance_;
	std::map<
		budgeting::BudgetItemIdentifier,
		std::set<accounting::Transaction>
	> usedBudget_;
	std::map<
		budgeting::BudgetItemIdentifier,
		pecunia::currency::Money
	> unusedBudget_;

public:
	BankAccountSurvey() = default;
	BankAccountSurvey(
		banking::ReconciledBankAccount reconciled,
		pecunia::currency::Money projectedBalance,
		std::map<
			budgeting::BudgetItemIdentifier,
			std::set<accounting::Transaction>
		> usedBudget,
		std::map<
			budgeting::BudgetItemIdentifier,
			pecunia::currency::Money
		> unusedBudget
	);
	const banking::ReconciledBankAccount& reconciled() const noexcept;
	const pecunia::currency::Money& projectedBalance() const noexcept;
	const std::map<
		budgeting::BudgetItemIdentifier,
		std::set<accounting::Transaction>
	>& usedBudget() const noexcept;
	const std::map<
		budgeting::BudgetItemIdentifier,
		pecunia::currency::Money
	>& unusedBudget() const noexcept;
};

}}

#endif
